--[[=============================================================================
    Receiver Protocol Initialization Functions

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.device_messages = "2015.03.31"
end


PROTOCOL_DECLARATIONS = {}

function ON_DRIVER_EARLY_INIT.receiver_init()

end

function ON_DRIVER_INIT.receiver_init()
	--LogTrace("ON_DRIVER_INIT.ProtocolDeclarations()")
	for k,v in pairs(PROTOCOL_DECLARATIONS) do
		if (PROTOCOL_DECLARATIONS[k] ~= nil and type(PROTOCOL_DECLARATIONS[k]) == "function") then
			PROTOCOL_DECLARATIONS[k]()
		end
	end	
end

function ON_DRIVER_LATEINIT.receiver_init()
end
	
function PROTOCOL_DECLARATIONS.CommandsTableInit_IR()
	LogTrace("PROTOCOL_DECLARATIONS.CommandsTableInit_IR()")

	CMDS_IR = {
		--index:  Proxy Command Name
		--value:  IR Code ID	 
		["ON"]             = "",
		["OFF"]            = "",
		["MUTE_ON"]        = "",
		["MUTE_OFF"]       = "",
		["MUTE_TOGGLE"]    = "",
		["INPUT_TOGGLE"]   = "",
		["NUMBER_0"]       = "", 	
		["NUMBER_1"]       = "",		
		["NUMBER_2"]       = "",	
		["NUMBER_3"]       = "",	
		["NUMBER_4"]       = "",	
		["NUMBER_5"]       = "",	
		["NUMBER_6"]       = "",	
		["NUMBER_7"]       = "",	
		["NUMBER_8"]       = "",	
		["NUMBER_9"]       = "",
		["STAR"]           = "",
		["DOT"]            = "",
		["INFO"]           = "", --Display
		["RECALL"]         = "",
		["PULSE_VOL_DOWN"] = "",
		["PULSE_VOL_UP"]   = "",
		["MENU"]           = "",
		["ENTER"]          = "",
		["UP"]             = "",
		["DOWN"]           = "",
		["LEFT"]           = "",
		["RIGHT"]          = "",
		["PREV"]           = "",
		["CANCEL"]         = "",
	}		
	
	--IR ZONE Commands
	CMDS_IR_ZONES = {}
	
	CMDS_IR_ZONES[0] = {
		--index:  Proxy Command Name
		--value:  IR Code ID
		["CONNECT_OUTPUT"] = "",
		["DISCONNECT_OUTPUT"] = "",	
		["VOLUME_UP"] = "",
		["VOLUME_DOWN"] = "",
		["MUTE_ON"] = "",
		["MUTE_OFF"] = "",
		["MUTE_TOGGLE"] = "",
	}	
	
	CMDS_IR_ZONES[1] = {
		["CONNECT_OUTPUT"] = "",
		["DISCONNECT_OUTPUT"] = "",
		["VOLUME_UP"] = "",
		["VOLUME_DOWN"] = "",
		["MUTE_ON"] = "",
		["MUTE_OFF"] = "",
		["MUTE_TOGGLE"] = "",
	}		
	
	CMDS_IR_ZONES[2] = {
		["CONNECT_OUTPUT"] = "",
		["DISCONNECT_OUTPUT"] = "",	
		["VOLUME_UP"] = "",
		["VOLUME_DOWN"] = "",
		["MUTE_ON"] = "",
		["MUTE_OFF"] = "",
		["MUTE_TOGGLE"] = "",		
	}	
	
	CMDS_IR_ZONES[3] = {
		["CONNECT_OUTPUT"] = "",
		["DISCONNECT_OUTPUT"] = "",	
		["VOLUME_UP"] = "",
		["VOLUME_DOWN"] = "",
		["MUTE_ON"] = "",
		["MUTE_OFF"] = "",
		["MUTE_TOGGLE"] = "",		
	}
	
	CMDS_IR[RECEIVER_PROXY_BINDINGID] = {}	
	
	CMDS_IR[TUNER_PROXY_BINDINGID] = {
		--values are IRCode ID as defined in IRCode XML declarations
		["PULSE_CH_UP"]="",
		["START_CH_UP"]="",		--used for PULSE_CH_UP in some Navigators
		["SKIP_FWD"]="",		--used for PULSE_CH_UP in some Navigators	
		["PULSE_CH_DOWN"]="",
		["START_CH_DOWN"]="",	--used for PULSE_CH_DOWN in some Navigators
		["SKIP_REV"]="",		--used for PULSE_CH_DOWN in some Navigators
		["PRESET_UP"]="",	
		["PRESET_DOWN"]="",
		["INPUT_TOGGLE"]="",	  --BAND Toggle NOT MAPPED?
		['PAGE_UP']="",		--Preset Up
		['PAGE_DOWN']="",	--Peeset Down	
		
		--these entries are only required if 
		--specific command codes are required for the Tuner
		["STAR"]="TAC",
		["NUMBER_0"]="", 	
		["NUMBER_1"]="",	
		["NUMBER_2"]="",	
		["NUMBER_3"]="",	
		["NUMBER_4"]="",	
		["NUMBER_5"]="",	
		["NUMBER_6"]="",	
		["NUMBER_7"]="",	
		["NUMBER_8"]="",	
		["NUMBER_9"]="",
		
		
	}		
end

function PROTOCOL_DECLARATIONS.CommandsTableInit_Serial()
	LogTrace("PROTOCOL_DECLARATIONS.CommandsTableInit_Serial()")

	CMDS = {
		--index:  Proxy Command Name
		--value:  Protocol Command Data
		
		--Power
		["ON"]             	= "",
		["OFF"]            	= "",
			
		--Menu
		["INFO"] 			= "INFO",		--INFO(TITLE)
		["GUIDE"] 		= "AUDIO",	--Displays the post processing page in the OSD User Menu
		["MENU"] 			= "MENU",
		["CANCEL"] 		= "EXIT",
		["UP"] 			= "DIRU",
		["DOWN"] 			= "DIRD",
		["LEFT"] 			= "DIRDL",
		["RIGHT"] 		= "DIRDR",
		["ENTER"] 		= "ENTER",	
		["RECALL"]         	= "BACK",
		["PREV"]           	= "BACK",
		["PLAY"]            = "PLAY",
          ["SKIP_FWD"]        = "NEXT",
	     ["SKIP_REV"]        = "PREV",

		
		--Digits
		["NUMBER_0"]		= "NUM(0)", 	
		["NUMBER_1"]       	= "NUM(1)",		
		["NUMBER_2"]       	= "NUM(2)",	
		["NUMBER_3"]       	= "NUM(3)",	
		["NUMBER_4"]       	= "NUM(4)",	
		["NUMBER_5"]       	= "NUM(5)",	
		["NUMBER_6"]       	= "NUM(6)",	
		["NUMBER_7"]       	= "NUM(7)",	
		["NUMBER_8"]       	= "NUM(8)",	
		["NUMBER_9"]       	= "NUM(9)",
		["STAR"]           	= "SETUP",
		["DOT"]            	= "SETUP",

	}

	CMDS[RECEIVER_PROXY_BINDINGID] = {}
	
	CMDS[TUNER_PROXY_BINDINGID] = {
		["PULSE_CH_UP"]="",
		["START_CH_UP"]="",		--used for PULSE_CH_UP in some Navigators
		["SKIP_FWD"]="",		--used for PULSE_CH_UP in some Navigators	
		["SCAN_FWD"]="",		
		["PULSE_CH_DOWN"]="",	--used for PULSE_CH_UP in some Navigators
		["START_CH_DOWN"]="",	--used for PULSE_CH_DOWN in some Navigators
		["SKIP_REV"]="",		--used for PULSE_CH_DOWN in some Navigators
		["SCAN_REV"]="",
		["PRESET_UP"]="",	
		["PRESET_DOWN"]="",
		["INPUT_TOGGLE"]="",	  --BAND Toggle NOT MAPPED?
		
		--these entries are only required if 
		--specific command strings are required for the Tuner
		["STAR"]="TAC",
		["NUMBER_0"]="", 	
		["NUMBER_1"]="",	
		["NUMBER_2"]="",	
		["NUMBER_3"]="",	
		["NUMBER_4"]="",	
		["NUMBER_5"]="",	
		["NUMBER_6"]="",	
		["NUMBER_7"]="",	
		["NUMBER_8"]="",	
		["NUMBER_9"]="",
	}	
	
end

function PROTOCOL_DECLARATIONS.InputOutputTableInit()
	LogTrace("PROTOCOL_DECLARATIONS.InputOutputTableInit()")
	----------------------------------------- [*COMMAND/RESPONSE HELPER TABLES*] -----------------------------------------
	
	tOutputCommandMap = {
		--index:  mod 1000 value of Output Connection id
		--value:  Protocol Command Data
		[0] = "",
		[1] = "Z",
	}
	
	tInputCommandMap = {
		--index:  Connection Name
		--value:  Protocol Command Data	
		--["ADAPTER PORT"] = "33", 
		["Digital 1 (AES)"] = "0",
		["Digital 2 (Coax)"] = "1",
		["Digital 3 (Coax)"] = "2",
		["Digital 4 (Opt.)"] = "3",
		["Digital 5 (Opt.)"] = "4",
		["Digital 6 (Opt.)"] = "5",
	     ["USB Input"] = "6",
		["HDMI 1"] = "8",
		["HDMI 2"] = "9",
		["HDMI 3"] = "10",
		["HDMI ARC"] = "11",
		["Analog 1"] = "12",
		["Analog 2"] = "13",
		["Analog 3"] = "14",
		["Analog 4"] = "15",
		["Analog 5"] = "16",
		["Analog 6 (XLR)"] = "17",
		["Preset0"] = "18",
		["Preset1"] = "19",
		["Preset2"] = "20",
		["Preset3"] = "21",
		["Preset4"] = "22",
		["Preset5"] = "23",
		["Preset6"] = "24",
		["Preset7"] = "25",
		["Preset8"] = "26",
		["Preset9"] = "27",
		["vTuner"] = "28",
		["Spotify"] = "29",
		["Airplay"] = "30",
		["Roon Ready"] = "31",
		["UPnP"] = "32",
		["USB File"] = "33",
		["Bluetooth"] = "34",
	}
	
	--SRC
	tInputResponseMap = {
		--index:  mod 1000 value of Input Connection id
	     ["0"] = "Digital 1 (AES)",
		["1"] = "Digital 2 (Coax)",
		["2"] = "Digital 3 (Coax)",
	     ["3"] = "Digital 4 (Opt.)",
		["4"] = "Digital 5 (Opt.)",
		["5"] = "Digital 6 (Opt.)",
		["6"] = "USB Input",
		["8"] = "HDMI 1",
		["9"] = "HDMI 2",
		["10"] = "HDMI 3",
		["11"] = "HDMI ARC",
	     ["12"] = "Analog 1", 
		["13"] = "Analog 2",
		["14"] = "Analog 3",
		["15"] = "Analog 4",
	     ["16"] = "Analog 5",
		["17"] = "Analog 6 (XLR)",
		["18"] = "Preset0",
		["19"] = "Preset1",
		["20"] = "Preset2",
		["21"] = "Preset3",
		["22"] = "Preset4",
		["23"] = "Preset5",
		["24"] = "Preset6",
		["25"] = "Preset7",
		["26"] = "Preset8",
		["27"] = "Preset9",
		["28"] = "vTuner",
		["29"] = "Spotify",
		["30"] = "Airplay",
		["31"] = "Roon Ready",
		["32"] = "UPnP",
		["33"] = "USB File", 
		["34"] = "Bluetooth", 
	}	
	
	
	tZoneBInputCommandMap = {
		["Digital 1 (AES)"] = "12",
		["Digital 2 (Coax)"] = "2",
		["Digital 3 (Coax)"] = "3",
		["Digital 4 (Opt.)"] = "4",
		["Digital 5 (Opt.)"] = "5",
		["Digital 6 (Opt.)"] = "6",
		["SPDIF 7 (Coaxial)"] = "7",
		["SPDIF 8 (Coaxial)"] = "8",
		["USB Input"] = "9",
		["Analog 1"] = "10",
		["Analog 2"] = "11",
		["Analog 3"] = "12",
		["Analog 4"] = "13",
		["Balanced 1"] = "14",
		["Balanced 2"] = "15",
		["Phono"] = "16",
	}
	
	tZoneBInputResponseMap = {
		--index:  mod 1000 value of Input Connection id
		["0"] = "Follow Main",
		["12"] = "Digital 1 (AES)",
		["2"] = "Digital 2 (Coax)",
		["3"] = "Digital 3 (Coax)",
		["4"] = "Digital 4 (Opt.)",
		["5"] = "Digital 5 (Opt.)",
		["6"] = "Digital 6 (Opt.)",
		["7"] = "SPDIF 7 (Coaxial)",
		["8"] = "SPDIF 8 (Coaxial)",
		["9"] = "USB Input", 
		["10"] = "Analog 1", 
		["11"] = "Analog 2",
		["120"] = "Analog 3",
		["13"] = "Analog 4",
		["14"] = "Balanced 1",
		["15"] = "Balanced 2",
		["16"] = "Phono", 
	}	
	
	
	tAudioInputResponseMap = {
		--index:  mod 1000 value of Input Connection id
		["0"] = "None",
		["1"] = "HDMI 1",
		["2"] = "8 Channel Analog",
		["12"] = "Digital 1 (AES)",
		["4"] = "Digital 2 (Coax)",
		["5"] = "Digital 3 (Coax)",
		["6"] = "Digital 4 (Opt.)",
		["7"] = "Digital 5 (Opt.)",
		["8"] = "Digital 6 (Opt.)",
		["9"] = "SPDIF 7 (Coaxial)",
		["10"] = "SPDIF 8 (Coaxial)",
		["120"] = "USB Input", 
		["13"] = "Analog 1", 
		["14"] = "Analog 2",
		["15"] = "Analog 3",
		["16"] = "Analog 4",
		["17"] = "Balanced 1",
		["18"] = "Balanced 2",
		["19"] = "Phono", 
		["21"] = "HDMI ARC",
	}
	
	tVideoInputResponseMap = {
		--index:  mod 1000 value of Input Connection id
		["0"] = "None",
		["1"] = "HDMI 1",
		["2"] = "HDMI 2",
		["3"] = "HDMI 3",
		["4"] = "HDMI 4",
		["5"] = "HDMI 5",
		["6"] = "HDMI 6",
		["7"] = "HDMI 7",
		["8"] = "HDMI 8",
		["9"] = "HDMI ARC",
	}	
	
	tInputCommandMap[TUNER_PROXY_BINDINGID] = {
	     ["FM Antenna"] = "",
		["AM Antenna"] = "",
	}		
	
	tInputCommandMap_IR = { 
	}
	--Main Zone Input Selection Commands	
	tInputCommandMap_IR[0] = { 
		--index:  Connection Name
		--value:  IR Code ID
		--["ADAPTER PORT"] = "51362",
		[""] = "",
	}
	--Zone2 Input Selection Commands
	tInputCommandMap_IR[1] = { 	
		[""] = "",
	}
	--Zone3 Input Selection Commands
	tInputCommandMap_IR[2] = { 
		[""] = "",
	}
	--Zone4 Input Selection Commands	
	tInputCommandMap_IR[3] = { 
		[""] = "",
	}	
	
	--Tuner Antennas
	tInputCommandMap_IR[TUNER_PROXY_BINDINGID] = {
		--values are IRCode ID as defined in IRCode XML declarations
		["AM Antenna"] = "",
		["FM Antenna"] = "", 
	}	
	
	----------------------------------------- [*I/O HELPER TABLES*] -----------------------------------------
	tOutputConnMap = {
		--index:  mod 1000 value of Output Connection id
		--value:  Output Connection Name
		[0] = "Main Output",
		[1] = "Zone B Output",
	}

	tInputConnMapByID = {
		--index:  mod 1000 value of Input Connection id
		--[0] = {Name = "INPUT HDMI 1",BindingID = RECEIVER_PROXY_BINDINGID,},
		--The first two entries below are for the antenna connections which are bound to the tuner proxy; the in values will need to be updated base on the connection ID.
		[0] = {Name = "Digital 1 (AES)",BindingID = RECEIVER_PROXY_BINDINGID,},
		[1] = {Name = "Digital 2 (Coax)",BindingID = RECEIVER_PROXY_BINDINGID,},
		[2] = {Name = "Digital 3 (Coax)",BindingID = RECEIVER_PROXY_BINDINGID,},
		[3] = {Name = "Digital 4 (Opt.)",BindingID = RECEIVER_PROXY_BINDINGID,},
		[4] = {Name = "Digital 5 (Opt.)",BindingID = RECEIVER_PROXY_BINDINGID,},
		[5] = {Name = "Digital 6 (Opt.)",BindingID = RECEIVER_PROXY_BINDINGID,},
		[6] = {Name = "USB Input",BindingID = RECEIVER_PROXY_BINDINGID,},
		[7] = {Name = "None",BindingID = RECEIVER_PROXY_BINDINGID,},
		[8] = {Name = "HDMI 1",BindingID = RECEIVER_PROXY_BINDINGID,},
		[9] = {Name = "HDMI 2",BindingID = RECEIVER_PROXY_BINDINGID,},
		[10] = {Name = "HDMI 3",BindingID = RECEIVER_PROXY_BINDINGID,},
		[11] = {Name = "HDMI ARC",BindingID = RECEIVER_PROXY_BINDINGID,},
		[12] = {Name = "Analog 1",BindingID = RECEIVER_PROXY_BINDINGID,},
		[13] = {Name = "Analog 2",BindingID = RECEIVER_PROXY_BINDINGID,},
		[14] = {Name = "Analog 3",BindingID = RECEIVER_PROXY_BINDINGID,},
		[15] = {Name = "Analog 4",BindingID = RECEIVER_PROXY_BINDINGID,},
		[16] = {Name = "Analog 5",BindingID = RECEIVER_PROXY_BINDINGID,},
		[17] = {Name = "Analog 6 (XLR)",BindingID = RECEIVER_PROXY_BINDINGID,},
		[18] = {Name = "Preset0",BindingID = RECEIVER_PROXY_BINDINGID,},
		[19] = {Name = "Preset1",BindingID = RECEIVER_PROXY_BINDINGID,},
		[20] = {Name = "Preset2",BindingID = RECEIVER_PROXY_BINDINGID,},
		[21] = {Name = "Preset3",BindingID = RECEIVER_PROXY_BINDINGID,},
		[22] = {Name = "Preset4",BindingID = RECEIVER_PROXY_BINDINGID,},
		[23] = {Name = "Preset5",BindingID = RECEIVER_PROXY_BINDINGID,},
		[24] = {Name = "Preset6",BindingID = RECEIVER_PROXY_BINDINGID,},
		[25] = {Name = "Preset7",BindingID = RECEIVER_PROXY_BINDINGID,},
		[26] = {Name = "Preset8",BindingID = RECEIVER_PROXY_BINDINGID,},
		[27] = {Name = "Preset9",BindingID = RECEIVER_PROXY_BINDINGID,},
		[28] = {Name = "vTuner",BindingID = RECEIVER_PROXY_BINDINGID,},
		[29] = {Name = "Spotify",BindingID = RECEIVER_PROXY_BINDINGID,},
		[30] = {Name = "Airplay",BindingID = RECEIVER_PROXY_BINDINGID,},
		[31] = {Name = "Roon Ready",BindingID = RECEIVER_PROXY_BINDINGID,},
		[32] = {Name = "UPnP",BindingID = RECEIVER_PROXY_BINDINGID,},
		[33] = {Name = "USB File",BindingID = RECEIVER_PROXY_BINDINGID,},
		[34] = {Name = "Bluetooth",BindingID = RECEIVER_PROXY_BINDINGID,},
	}
	
	tInputConnMapByName = {
		--index:  Input Connection Name
		--ID: mod 1000 value of Input Connection id
		--["INPUT HDMI 1"] = {ID = 0,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Digital 1 (AES)"] = {ID = 0,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Digital 2 (Coax)"] = {ID = 1,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Digital 3 (Coax)"] = {ID = 2,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Digital 4 (Opt.)"] = {ID = 3,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Digital 5 (Opt.)"] = {ID = 4,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Digital 6 (Opt.)"] = {ID = 5,BindingID = RECEIVER_PROXY_BINDINGID,},
		["USB Input"] = {ID = 6,BindingID = RECEIVER_PROXY_BINDINGID,},
		["None"] = {ID = 7,BindingID = RECEIVER_PROXY_BINDINGID,},
		["HDMI 1"] = {ID = 8,BindingID = RECEIVER_PROXY_BINDINGID,},
		["HDMI 2"] = {ID = 9,BindingID = RECEIVER_PROXY_BINDINGID,},
		["HDMI 3"] = {ID = 10,BindingID = RECEIVER_PROXY_BINDINGID,},
		["HDMI ARC"] = {ID = 11,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Analog 1"] = {ID = 12,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Analog 2"] = {ID = 13,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Analog 3"] = {ID = 14,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Analog 4"] = {ID = 15,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Analog 5"] = {ID = 16,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Analog 6 (XLR)"] = {ID = 17,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Preset0"] = {ID = 18,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Preset1"] = {ID = 19,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Preset2"] = {ID = 20,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Preset3"] = {ID = 21,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Preset4"] = {ID = 22,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Preset5"] = {ID = 23,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Preset6"] = {ID = 24,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Preset7"] = {ID = 25,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Preset8"] = {ID = 26,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Preset9"] = {ID = 27,BindingID = RECEIVER_PROXY_BINDINGID,},
		["vTuner"] = {ID = 28,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Spotify"] = {ID = 29,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Airplay"] = {ID = 30,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Roon Ready"] = {ID = 31,BindingID = RECEIVER_PROXY_BINDINGID,},
		["UPnP"] = {ID = 32,BindingID = RECEIVER_PROXY_BINDINGID,},
		["USB File"] = {ID = 33,BindingID = RECEIVER_PROXY_BINDINGID,},
		["Bluetooth"] = {ID = 34,BindingID = RECEIVER_PROXY_BINDINGID,},
	}

end	

function PROTOCOL_DECLARATIONS.PowerCommandsTableInit_Serial()
	LogTrace("PROTOCOL_DECLARATIONS.PowerCommandsTableInit_Serial()")
	
	tPowerCommandMap = {
		--index:  mod 1000 value of Output Connection id
		--value:  Protocol Command Data (Power)
		[0] = "MAIN",
		[1] = "ZONE2",
	}
	
	tPowerQueryMap = {
		--index:  mod 1000 value of Output Connection id
		--value:  Protocol Command Data (Power)
		[0] = "PWR",
		[1] = "POWERZONE2",
	}
end

function PROTOCOL_DECLARATIONS.VolumeCommandsTableInit_Serial()
	LogTrace("PROTOCOL_DECLARATIONS.VolumeCommandsTableInit_Serial()")
	
	tVolumeCommandMap = {
		--index:  mod 1000 value of Output Connection id
		--value:  Protocol Command Data (Pulse Volume - command prefix/suffix)
		[0] = "VOL",
		[1] = "ZVOL",	
	}

	tVolumeSetCommandMap = {
		--index:  mod 1000 value of Output Connection id
		--value:  Protocol Command Data (Discreet Volume - command prefix/suffix)	
		[0] = "VOL",
		[1] = "ZVOL",	
	}
	
	tVolumeQueryMap = {
		--index:  mod 1000 value of Output Connection id
		--value:  Protocol Command Data (Volume Query)		
		[0] = "VOL",
		[1] = "ZVOL",	
	}	

	tMuteCommandMap = {
		--index:  mod 1000 value of Output Connection id
		--value:  Protocol Command Data (Mute - command prefix/suffix)		
		[0] = "MUTE",
		[1] = "ZMUTE",
	}	

	
	--[[
	TODO: Update tVolumeCurve table based upon devices volume range and optimal ramping steps
		tVolumeCurve is used to plot values that will be used
		to build volume commands during volume ramping.
		Specifically, they are used in GetNextVolumeCurveValue() which is 
		called from the ContinueVolumeRamping() function. 
		The values entered here should correlate to the volume parameters as defined in the device protocol.
		Below is an example, the table for your device may use different values.
		Also the number and size of steps may differ for your device. Adjust values to get 
		a smooth volume ramping from low to high volume.
		NOTE: values must be in decimal (i.e. not hexadecimal...)
	--]]
	tVolumeCurve = {
		-999, -813, -704, -627, -566,
	     -517, -476, -440, -408, -354,
	     -380, -331, -309, -289, -254,
	     -271, -237, -222, -207, -194,
	     -180, -168, -156, -144, -133,
	     -123, -113, -103, -94, -84,
	     -76, -67, -59, -51, -43, 
	     -35, -28, -21, -14, -7,
	     0, 6, 12, 19, 36,
	     25, 42, 31, 48, 53,
	     58, 64, 69, 74, 79,
	     84, 88, 93, 98, 102,
	     107, 111, 115, 119, 120,
		--107, 111, 115, 119, 124,
	     --128, 132, 136, 140, 144,
	     --147, 151, 155, 162, 158,
	     --166, 169, 173, 176, 179,
	     --183, 186, 189, 193, 196,
	     --199, 202, 205, 208, 211,
	     --214, 217, 220, 223, 229,
	     --226, 231, 234, 237, 240,
	}
	
end

function PROTOCOL_DECLARATIONS.SurroundModes()
	LogTrace("PROTOCOL_DECLARATIONS.SurroundModes()")
	
	--index:  value of Surround Mode id
	--value:  Protocol Command Data
	tSurroundModes = {
		[23303] = "0001", -- STEREO (cyclic)
		[23304] = "0003", -- FRONT STAGE SURROUND ADVANCE
	}

	tSurroundResponses = {
		--index:  Protocol Command/Response Data 
		--value:  {ID = value of Surround Mode id, Name = "Name of Surround Mode"
		--["0001"] = { ID = 23303, Name = "STEREO (cyclic)"},
		["0001"] = { ID = 23303, Name = "STEREO (cyclic)"},
		["0003"] = { ID = 23304, Name = "FRONT STAGE SURROUND ADVANCE"},
	}
	
	tSurroundModes_IR = {
		--index:  Pvalue of Surround Mode id
		--value:  IR Code ID
		[23303] = "51277", -- STEREO (cyclic)
		[23304] = "51322", -- FRONT STAGE SURROUND ADVANCE
	}	
end

function PROTOCOL_DECLARATIONS.Tuner()
	LogTrace("PROTOCOL_DECLARATIONS.Tuner()")
	
	--INT"L TUNING:Global Table
	tFreqStepInfo = {
		["FM50/AM9"] =   { FmMin = 8750, FmMax = 10800, FmSpacing = 5,  AmMin = 531, AmMax = 1611, AmSpacing = 9,  },
		["FM100/AM9"] =  { FmMin = 7600, FmMax = 9000,  FmSpacing = 10, AmMin = 531, AmMax = 1611, AmSpacing = 9,  },
		["FM100/AM10"] = { FmMin = 8750, FmMax = 10800, FmSpacing = 10, AmMin = 530, AmMax = 1710, AmSpacing = 10, },
		["FM200/AM10"] = { FmMin = 8750, FmMax = 10790, FmSpacing = 20, AmMin = 530, AmMax = 1710, AmSpacing = 10, },
	}	

	tTunerPresetCommandMap = {
		--index:  mod 1000 value of Output Connection id
		--value:  Protocol Command Data (Pulse Tuner Preset - command prefix/suffix)
		[0] = "",
		[1] = "",
	}

	tTunerTuningCommandMap = {
		--index:  mod 1000 value of Output Connection id
		--value:  Protocol Command Data (Pulse Tuner Frequency - command prefix/suffix)
		[0] = "",
		[1] = "",
	}
	
end

function ReverseTable(a)
	local b = {}
	for k,v in pairs(a) do b[v] = k end
	return b
end

